// Scene.cpp: implementation of the Scene class.
//
//////////////////////////////////////////////////////////////////////

#include "Scene.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Scene::Scene()
{
	WriteCaps();
	InitDX();
	if(fullscreen)ShowCursor(0);
}

Scene::~Scene()
{
	if(fullscreen)ShowCursor(1);
}

static unsigned char r,g,b;
static unsigned char rp=1,gp=1,bp=1;

void Scene::DrawDXScene()
{
    if( NULL == pd )return;

	int random = rand();
	if(random%3==0)
	{
		if(rp)r++;
		else r--;
		if(r==0 || r==255)rp=!rp;
	}
	if(random%3==1)	
	{
		if(gp)g++;
		else g--;
		if(g==0 || g==255)gp=!gp;
	}
	if(random%3==2)
	{
		if(bp)b++;
		else b--;
		if(b==0 || b==255)bp=!bp;
	}

    // Clear the backbuffer to a blue color
	pd->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(r,g,b), 1.0f, 0 );

    // Begin the scene
    pd->BeginScene();

	// End the scene
	pd->EndScene();
	if(keys[VK_F12])SaveScreen_BMP();

	// Present the backbuffer contents to the display
	pd->Present( NULL, NULL, NULL, NULL );
}

#ifndef PI180
#define PI180	0.0174532925199432957692369076848861f	// pi / 180
#endif

void Scene::ReSizeDXScene(int width, int height)
{
	D3DXMATRIX matProj;
	D3DXMatrixPerspectiveFovLH( &matProj, FIELD_OF_VIEW*PI180, (float)width/(float)height, Z_NEAR_VIEW_PLANE, Z_FAR_VIEW_PLANE );
	pd->SetTransform( D3DTS_PROJECTION, &matProj );
}

void Scene::InitDX()
{
	if(!fullscreen)
	{
		RECT		WindowRect;
		WindowRect.left=(long)0;
		WindowRect.top=(long)0;
		GetClientRect(hWnd,&WindowRect);
		ReSizeDXScene(WindowRect.right,WindowRect.bottom);	// set perspective
	}
	else ReSizeDXScene(screen_x, screen_y);	//definovana v tomto subore

	pd->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE );
	
	// Enable z-buffering.
	pd->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
	pd->SetRenderState(D3DRS_ZFUNC, D3DCMP_LESS);

//	mat4 m;
//	pd->SetTransform( D3DTS_VIEW, (struct _D3DMATRIX *)(void*)&m );
//	pd->SetTransform( D3DTS_WORLD, (struct _D3DMATRIX *)(void*)&m );
}

void Scene::WriteCaps()
{
	FILE* f;

	D3DADAPTER_IDENTIFIER8 indentif;
	D3DCAPS8 caps;
	pD3D->GetAdapterIdentifier( D3DADAPTER_DEFAULT, 0, &indentif);
	pd->GetDeviceCaps( &caps);

	f = fopen("D3D8caps.txt","wb");
	if(f==NULL)return;

//	D3DADAPTER_IDENTIFIER8
	fprintf(f,"Adapter\r\n");
	fprintf(f,"Driver: %s\r\n", indentif.Driver);
	fprintf(f,"Description: %s\r\n", indentif.Description);

	fprintf(f,"Product: %x\t", HIWORD(indentif.DriverVersion.HighPart));
	fprintf(f,"Version: %x\t", LOWORD(indentif.DriverVersion.HighPart));
	fprintf(f,"SubVersion: %x\t\t", HIWORD(indentif.DriverVersion.LowPart));
	fprintf(f,"Build: %x\r\n", LOWORD(indentif.DriverVersion.LowPart));

	fprintf(f,"VendorId: %x\t", indentif.VendorId);
	fprintf(f,"DeviceId: %x\t", indentif.DeviceId);
	fprintf(f,"SubSysId: %x\t", indentif.SubSysId);
	fprintf(f,"Revision: %x\r\n", indentif.Revision);

//	D3DDEVTYPE		DeviceType;
	if(caps.DeviceType == D3DDEVTYPE_HAL)	fprintf(f,"Typ ovladaca: HAL\r\n");
	if(caps.DeviceType == D3DDEVTYPE_REF)	fprintf(f,"Typ ovladaca: REF\r\n");
	if(caps.DeviceType == D3DDEVTYPE_SW)	fprintf(f,"Typ ovladaca: SW\r\n");

//	DWORD               VertexShaderVersion;
	fprintf(f,"\r\nVertex Shader\r\n");
	fprintf(f,"VertexShaderVersion : \t\t%d.%d\r\n",HIBYTE(LOWORD(caps.VertexShaderVersion)),LOBYTE(LOWORD(caps.VertexShaderVersion)));
//	DWORD               MaxVertexShaderConst;
	fprintf(f,"MaxVertexShaderConst : \t\t%d\r\n",caps.MaxVertexShaderConst);
	if(software_vertexprocessing)	fprintf(f,"Hardware_VertexProcessing : \tNO\r\n");
	else							fprintf(f,"Hardware_VertexProcessing : \tYES\r\n");

	fprintf(f,"\r\nPixel Shader\r\n");
//	DWORD               PixelShaderVersion;
	fprintf(f,"PixelShaderVersion : \t\t%d.%d\r\n",HIBYTE(LOWORD(caps.PixelShaderVersion)),LOBYTE(LOWORD(caps.PixelShaderVersion)));
//	DWORD               MaxPixelShaderValue;
	fprintf(f,"MaxPixelShaderValue : \t\t%f\r\n\r\n",caps.MaxPixelShaderValue);

//	DWORD				MaxTextureBlendStages;
	fprintf(f,"MaxTextureBlendStages : \t%d\r\n",caps.MaxTextureBlendStages);
//	DWORD				MaxSimultaneousTextures;
	fprintf(f,"MaxSimultaneousTextures : \t%d\r\n",caps.MaxSimultaneousTextures);
//	DWORD               TextureOpCaps;
	fprintf(f,"TextureOpCaps\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_ADD )				fprintf(f,"ADD : \t\t\t\tYES\r\n");
	else													fprintf(f,"ADD : \t\t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_ADDSIGNED )			fprintf(f,"ADDSIGNED : \t\t\tYES\r\n");
	else													fprintf(f,"ADDSIGNED : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_ADDSIGNED2X )		fprintf(f,"ADDSIGNED2X : \t\t\tYES\r\n");
	else													fprintf(f,"ADDSIGNED2X : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_ADDSMOOTH )			fprintf(f,"ADDSMOOTH : \t\t\tYES\r\n");
	else													fprintf(f,"ADDSMOOTH : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_BLENDCURRENTALPHA )	fprintf(f,"BLENDCURRENTALPHA : \t\tYES\r\n");
	else													fprintf(f,"BLENDCURRENTALPHA : \t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_BLENDDIFFUSEALPHA )	fprintf(f,"BLENDDIFFUSEALPHA : \t\tYES\r\n");
	else													fprintf(f,"BLENDDIFFUSEALPHA : \t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_BLENDFACTORALPHA )	fprintf(f,"BLENDFACTORALPHA : \t\tYES\r\n");
	else													fprintf(f,"BLENDFACTORALPHA : \t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_BLENDTEXTUREALPHA )	fprintf(f,"BLENDTEXTUREALPHA : \t\tYES\r\n");
	else													fprintf(f,"BLENDTEXTUREALPHA : \t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_BLENDTEXTUREALPHAPM )fprintf(f,"BLENDTEXTUREALPHAPM : \t\tYES\r\n");
	else													fprintf(f,"BLENDTEXTUREALPHAPM : \t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_BUMPENVMAP )			fprintf(f,"BUMPENVMAP : \t\t\tYES\r\n");
	else													fprintf(f,"BUMPENVMAP : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_BUMPENVMAPLUMINANCE )fprintf(f,"BUMPENVMAPLUMINANCE : \t\tYES\r\n");
	else													fprintf(f,"BUMPENVMAPLUMINANCE : \t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_DISABLE )			fprintf(f,"DISABLE : \t\t\tYES\r\n");
	else													fprintf(f,"DISABLE : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_DOTPRODUCT3 )		fprintf(f,"DOTPRODUCT3 : \t\t\tYES\r\n");
	else													fprintf(f,"DOTPRODUCT3 : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_LERP )				fprintf(f,"LERP : \t\t\t\tYES\r\n");
	else													fprintf(f,"LERP : \t\t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_MODULATE )			fprintf(f,"MODULATE : \t\t\tYES\r\n");
	else													fprintf(f,"MODULATE : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_MODULATE2X )			fprintf(f,"MODULATE2X : \t\t\tYES\r\n");
	else													fprintf(f,"MODULATE2X : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_MODULATE4X )			fprintf(f,"MODULATE4X : \t\t\tYES\r\n");
	else													fprintf(f,"MODULATE4X : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_MODULATEALPHA_ADDCOLOR )	fprintf(f,"MODULATEALPHA_ADDCOLOR : \tYES\r\n");
	else														fprintf(f,"MODULATEALPHA_ADDCOLOR : \tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_MODULATECOLOR_ADDALPHA )	fprintf(f,"MODULATECOLOR_ADDALPHA : \tYES\r\n");
	else														fprintf(f,"MODULATECOLOR_ADDALPHA : \tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_MODULATEINVALPHA_ADDCOLOR )	fprintf(f,"MODULATEINVALPHA_ADDCOLOR : \tYES\r\n");
	else															fprintf(f,"MODULATEINVALPHA_ADDCOLOR : \tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_MODULATEINVCOLOR_ADDALPHA )	fprintf(f,"MODULATEINVCOLOR_ADDALPHA : \tYES\r\n");
	else															fprintf(f,"MODULATEINVCOLOR_ADDALPHA : \tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_MULTIPLYADD )		fprintf(f,"MULTIPLYADD : \t\t\tYES\r\n");
	else													fprintf(f,"MULTIPLYADD : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_PREMODULATE )		fprintf(f,"PREMODULATE : \t\t\tYES\r\n");
	else													fprintf(f,"PREMODULATE : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_SELECTARG1 )			fprintf(f,"SELECTARG1 : \t\t\tYES\r\n");
	else													fprintf(f,"SELECTARG1 : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_SELECTARG2 )			fprintf(f,"SELECTARG2 : \t\t\tYES\r\n");
	else													fprintf(f,"SELECTARG2 : \t\t\tNO\r\n");
	if(caps.TextureOpCaps&D3DTEXOPCAPS_SUBTRACT )			fprintf(f,"SUBTRACT : \t\t\tYES\r\n\r\n");
	else													fprintf(f,"SUBTRACT : \t\t\tNO\r\n\r\n");

//	DWORD				MaxTextureWidth, MaxTextureHeight;
	fprintf(f,"MaxTextureWidth : \t\t%d\r\n",caps.MaxTextureWidth);
	fprintf(f,"MaxTextureHeight : \t\t%d\r\n",caps.MaxTextureHeight);
//	DWORD				MaxVolumeExtent;
	fprintf(f,"MaxVolumeExtent : \t\t%d\r\n",caps.MaxVolumeExtent);
//	DWORD				MaxTextureRepeat;
	fprintf(f,"MaxTextureRepeat : \t\t%d\r\n",caps.MaxTextureRepeat);
//	DWORD				MaxTextureAspectRatio;
	fprintf(f,"MaxTextureAspectRatio : \t%d\r\n",caps.MaxTextureAspectRatio);
//	DWORD				MaxAnisotropy;
	fprintf(f,"MaxAnisotropy : \t\t%d\r\n",caps.MaxAnisotropy);
//	float				MaxVertexW;
	fprintf(f,"MaxVertexW : \t\t\t%e\r\n",caps.MaxVertexW);

//	DWORD				MaxActiveLights;
	fprintf(f,"\r\nMaxActiveLights : \t\t%u\r\n",caps.MaxActiveLights);
//	DWORD				MaxUserClipPlanes;
	fprintf(f,"MaxUserClipPlanes : \t\t%d\r\n",caps.MaxUserClipPlanes);
//	DWORD				MaxVertexBlendMatrices;
	fprintf(f,"MaxVertexBlendMatrices : \t%d\r\n",caps.MaxVertexBlendMatrices);
//	DWORD				MaxVertexBlendMatrixIndex;
	fprintf(f,"MaxVertexBlendMatrixIndex : \t%d\r\n",caps.MaxVertexBlendMatrixIndex);

//	float				MaxPointSize;
	fprintf(f,"MaxPointSize : \t\t\t%f\r\n",caps.MaxPointSize);
//	DWORD				MaxPrimitiveCount;
	fprintf(f,"MaxPrimitiveCount : \t\t%d\r\n",caps.MaxPrimitiveCount);
//	DWORD				MaxVertexIndex;
	fprintf(f,"MaxVertexIndex : \t\t%d\r\n",caps.MaxVertexIndex);
//	DWORD				MaxStreams;
	fprintf(f,"MaxStreams : \t\t\t%d\r\n",caps.MaxStreams);
//	DWORD				MaxStreamStride;
	fprintf(f,"MaxStreamStride : \t\t%d\r\n",caps.MaxStreamStride);

//	UINT			AdapterOrdinal;

//	DWORD			Caps;
	if(caps.Caps==D3DCAPS_READ_SCANLINE)fprintf(f,"\r\nD3DCAPS_READ_SCANLINE: \t\t\tYES\r\n");
	else								fprintf(f,"\r\nD3DCAPS_READ_SCANLINE: \t\t\tNO\r\n");

//	DWORD			Caps2;
	if(caps.Caps2&D3DCAPS2_CANCALIBRATEGAMMA )	fprintf(f,"D3DCAPS2_CANCALIBRATEGAMMA : \t\tYES\r\n");
	else										fprintf(f,"D3DCAPS2_CANCALIBRATEGAMMA : \t\tNO\r\n");

	if(caps.Caps2&D3DCAPS2_CANRENDERWINDOWED )	fprintf(f,"D3DCAPS2_CANRENDERWINDOWED : \t\tYES\r\n");
	else										fprintf(f,"D3DCAPS2_CANRENDERWINDOWED : \t\tNO\r\n");

	if(caps.Caps2&D3DCAPS2_CANMANAGERESOURCE  )	fprintf(f,"D3DCAPS2_CANMANAGERESOURCE  : \t\tYES\r\n");
	else										fprintf(f,"D3DCAPS2_CANMANAGERESOURCE  : \t\tNO\r\n");

	if(caps.Caps2&D3DCAPS2_DYNAMICTEXTURES )	fprintf(f,"D3DCAPS2_DYNAMICTEXTURES : \t\tYES\r\n");
	else										fprintf(f,"D3DCAPS2_DYNAMICTEXTURES : \t\tNO\r\n");

	if(caps.Caps2&D3DCAPS2_FULLSCREENGAMMA )	fprintf(f,"D3DCAPS2_FULLSCREENGAMMA : \t\tYES\r\n");
	else										fprintf(f,"D3DCAPS2_FULLSCREENGAMMA : \t\tNO\r\n");

	if(caps.Caps2&D3DCAPS2_NO2DDURING3DSCENE )	fprintf(f,"D3DCAPS2_NO2DDURING3DSCENE : \t\tYES\r\n");
	else										fprintf(f,"D3DCAPS2_NO2DDURING3DSCENE : \t\tNO\r\n");

//	DWORD				Caps3;
	if(caps.Caps3&D3DCAPS3_ALPHA_FULLSCREEN_FLIP_OR_DISCARD )	fprintf(f,"D3DCAPS3_ALPHA_FULLSCREEN_FLIP_OR_DISCARD : YES\r\n");
	else										fprintf(f,"D3DCAPS3_ALPHA_FULLSCREEN_FLIP_OR_DISCARD : NO\r\n");

//	DWORD				PresentationIntervals;
	if(caps.PresentationIntervals&D3DPRESENT_INTERVAL_IMMEDIATE )	fprintf(f,"D3DPRESENT_INTERVAL_IMMEDIATE : \tYES\r\n");
	else															fprintf(f,"D3DPRESENT_INTERVAL_IMMEDIATE : \tNO\r\n");
	if(caps.PresentationIntervals&D3DPRESENT_INTERVAL_ONE )	fprintf(f,"D3DPRESENT_INTERVAL_ONE : \t\tYES\r\n");
	else													fprintf(f,"D3DPRESENT_INTERVAL_ONE : \t\tNO\r\n");
	if(caps.PresentationIntervals&D3DPRESENT_INTERVAL_TWO )	fprintf(f,"D3DPRESENT_INTERVAL_TWO : \t\tYES\r\n");
	else													fprintf(f,"D3DPRESENT_INTERVAL_TWO : \t\tNO\r\n");
	if(caps.PresentationIntervals&D3DPRESENT_INTERVAL_THREE )	fprintf(f,"D3DPRESENT_INTERVAL_THREE : \t\tYES\r\n");
	else														fprintf(f,"D3DPRESENT_INTERVAL_THREE : \t\tNO\r\n");
	if(caps.PresentationIntervals&D3DPRESENT_INTERVAL_FOUR )	fprintf(f,"D3DPRESENT_INTERVAL_FOUR : \t\tYES\r\n");
	else														fprintf(f,"D3DPRESENT_INTERVAL_FOUR : \t\tNO\r\n");

//	DWORD				CursorCaps;

//	DWORD				DevCaps;
	fprintf(f,"DevCaps\r\n");
	if(caps.DevCaps&D3DDEVCAPS_CANBLTSYSTONONLOCAL )	fprintf(f,"D3DDEVCAPS_CANBLTSYSTONONLOCAL : \tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_CANBLTSYSTONONLOCAL : \tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_CANRENDERAFTERFLIP )	fprintf(f,"D3DDEVCAPS_CANRENDERAFTERFLIP : \tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_CANRENDERAFTERFLIP : \tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_DRAWPRIMTLVERTEX )	fprintf(f,"D3DDEVCAPS_DRAWPRIMTLVERTEX : \t\tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_DRAWPRIMTLVERTEX : \t\tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_EXECUTESYSTEMMEMORY )fprintf(f,"D3DDEVCAPS_EXECUTESYSTEMMEMORY : \tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_EXECUTESYSTEMMEMORY : \tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_EXECUTEVIDEOMEMORY )	fprintf(f,"D3DDEVCAPS_EXECUTEVIDEOMEMORY : \tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_EXECUTEVIDEOMEMORY : \tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_HWRASTERIZATION  )	fprintf(f,"D3DDEVCAPS_HWRASTERIZATION : \t\tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_HWRASTERIZATION : \t\tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_HWTRANSFORMANDLIGHT )fprintf(f,"D3DDEVCAPS_HWTRANSFORMANDLIGHT : \tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_HWTRANSFORMANDLIGHT : \tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_NPATCHES  )			fprintf(f,"D3DDEVCAPS_NPATCHES : \t\t\tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_NPATCHES : \t\t\tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_PUREDEVICE  )		fprintf(f,"D3DDEVCAPS_PUREDEVICE : \t\tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_PUREDEVICE : \t\tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_QUINTICRTPATCHES  )	fprintf(f,"D3DDEVCAPS_QUINTICRTPATCHES : \t\tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_QUINTICRTPATCHES : \t\tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_RTPATCHES  )			fprintf(f,"D3DDEVCAPS_RTPATCHES : \t\t\tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_RTPATCHES : \t\t\tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_RTPATCHHANDLEZERO  )	fprintf(f,"D3DDEVCAPS_RTPATCHHANDLEZERO : \t\tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_RTPATCHHANDLEZERO : \t\tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_SEPARATETEXTUREMEMORIES   )fprintf(f,"D3DDEVCAPS_SEPARATETEXTUREMEMORIES  : \tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_SEPARATETEXTUREMEMORIES  : \tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_TEXTURENONLOCALVIDMEM  )	fprintf(f,"D3DDEVCAPS_TEXTURENONLOCALVIDMEM : \tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_TEXTURENONLOCALVIDMEM : \tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_TEXTURESYSTEMMEMORY   )	fprintf(f,"D3DDEVCAPS_TEXTURESYSTEMMEMORY : \tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_TEXTURESYSTEMMEMORY : \tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_TEXTUREVIDEOMEMORY   )	fprintf(f,"D3DDEVCAPS_TEXTUREVIDEOMEMORY : \tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_TEXTUREVIDEOMEMORY : \tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_TLVERTEXSYSTEMMEMORY   )	fprintf(f,"D3DDEVCAPS_TLVERTEXSYSTEMMEMORY : \tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_TLVERTEXSYSTEMMEMORY : \tNO\r\n");
	if(caps.DevCaps&D3DDEVCAPS_TLVERTEXVIDEOMEMORY   )	fprintf(f,"D3DDEVCAPS_TLVERTEXVIDEOMEMORY : \tYES\r\n");
	else											fprintf(f,"D3DDEVCAPS_TLVERTEXVIDEOMEMORY : \tNO\r\n");

//	DWORD				PrimitiveMiscCaps;
	fprintf(f,"PrimitiveMiscCaps\r\n");
	if(caps.PrimitiveMiscCaps&D3DPMISCCAPS_BLENDOP   )	fprintf(f,"D3DPMISCCAPS_BLENDOP : \t\t\tYES\r\n");
	else											fprintf(f,"D3DPMISCCAPS_BLENDOP : \t\t\tNO\r\n");
	if(caps.PrimitiveMiscCaps&D3DPMISCCAPS_CLIPPLANESCALEDPOINTS   )	fprintf(f,"D3DPMISCCAPS_CLIPPLANESCALEDPOINTS : \tYES\r\n");
	else											fprintf(f,"D3DPMISCCAPS_CLIPPLANESCALEDPOINTS : \tNO\r\n");
	if(caps.PrimitiveMiscCaps&D3DPMISCCAPS_CLIPTLVERTS   )	fprintf(f,"D3DPMISCCAPS_CLIPTLVERTS : \t\tYES\r\n");
	else											fprintf(f,"D3DPMISCCAPS_CLIPTLVERTS : \t\tNO\r\n");
	if(caps.PrimitiveMiscCaps&D3DPMISCCAPS_COLORWRITEENABLE   )	fprintf(f,"D3DPMISCCAPS_COLORWRITEENABLE : \tYES\r\n");
	else											fprintf(f,"D3DPMISCCAPS_COLORWRITEENABLE : \tNO\r\n");
	if(caps.PrimitiveMiscCaps&D3DPMISCCAPS_LINEPATTERNREP   )	fprintf(f,"D3DPMISCCAPS_LINEPATTERNREP : \t\tYES\r\n");
	else											fprintf(f,"D3DPMISCCAPS_LINEPATTERNREP : \t\tNO\r\n");
	if(caps.PrimitiveMiscCaps&D3DPMISCCAPS_MASKZ   )	fprintf(f,"D3DPMISCCAPS_MASKZ : \t\t\tYES\r\n");
	else											fprintf(f,"D3DPMISCCAPS_MASKZ : \t\t\tNO\r\n");
	if(caps.PrimitiveMiscCaps&D3DPMISCCAPS_TSSARGTEMP   )	fprintf(f,"D3DPMISCCAPS_TSSARGTEMP : \t\tYES\r\n");
	else											fprintf(f,"D3DPMISCCAPS_TSSARGTEMP : \t\tNO\r\n");
//	D3DPMISCCAPS_CULLCCW 
//	D3DPMISCCAPS_CULLCW 
//	D3DPMISCCAPS_CULLNONE 

//	DWORD				RasterCaps;
	fprintf(f,"RasterCaps\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_ANISOTROPY )	fprintf(f,"D3DPRASTERCAPS_ANISOTROPY : \t\tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_ANISOTROPY : \t\tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_ANTIALIASEDGES )	fprintf(f,"D3DPRASTERCAPS_ANTIALIASEDGES : \tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_ANTIALIASEDGES : \tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_COLORPERSPECTIVE )	fprintf(f,"D3DPRASTERCAPS_COLORPERSPECTIVE : \tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_COLORPERSPECTIVE : \tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_DITHER )	fprintf(f,"D3DPRASTERCAPS_DITHER : \t\tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_DITHER : \t\tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_FOGRANGE )	fprintf(f,"D3DPRASTERCAPS_FOGRANGE : \t\tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_FOGRANGE : \t\tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_FOGTABLE )	fprintf(f,"D3DPRASTERCAPS_FOGTABLE : \t\tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_FOGTABLE : \t\tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_FOGVERTEX )	fprintf(f,"D3DPRASTERCAPS_FOGVERTEX : \t\tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_FOGVERTEX : \t\tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_MIPMAPLODBIAS )	fprintf(f,"D3DPRASTERCAPS_MIPMAPLODBIAS : \t\tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_MIPMAPLODBIAS : \t\tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_PAT )	fprintf(f,"D3DPRASTERCAPS_PAT : \t\t\tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_PAT : \t\t\tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_STRETCHBLTMULTISAMPLE )	fprintf(f,"D3DPRASTERCAPS_STRETCHBLTMULTISAMPLE : \tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_STRETCHBLTMULTISAMPLE : \tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_WBUFFER )	fprintf(f,"D3DPRASTERCAPS_WBUFFER : \t\tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_WBUFFER : \t\tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_WFOG )	fprintf(f,"D3DPRASTERCAPS_WFOG : \t\t\tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_WFOG : \t\t\tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_ZBIAS )	fprintf(f,"D3DPRASTERCAPS_ZBIAS : \t\t\tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_ZBIAS : \t\t\tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_ZBUFFERLESSHSR )	fprintf(f,"D3DPRASTERCAPS_ZBUFFERLESSHSR : \tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_ZBUFFERLESSHSR : \tNO\r\n");
	if(caps.RasterCaps&D3DPRASTERCAPS_ZFOG )	fprintf(f,"D3DPRASTERCAPS_ZFOG : \t\t\tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_ZFOG : \t\t\tNO\r\n"); 
	if(caps.RasterCaps&D3DPRASTERCAPS_ZTEST )	fprintf(f,"D3DPRASTERCAPS_ZTEST : \t\t\tYES\r\n");
	else										fprintf(f,"D3DPRASTERCAPS_ZTEST : \t\t	\tNO\r\n");
 
// 	DWORD				ZCmpCaps;
	fprintf(f,"ZCmpCaps : \t\t\t\t%x\r\n",caps.ZCmpCaps);
//	DWORD				SrcBlendCaps;
	fprintf(f,"SrcBlendCaps : \t\t\t\t%x\r\n",caps.SrcBlendCaps);
//	DWORD				DestBlendCaps;
	fprintf(f,"DestBlendCaps : \t\t\t%x\r\n",caps.DestBlendCaps);
//	DWORD				AlphaCmpCaps;
	fprintf(f,"AlphaCmpCaps : \t\t\t\t%x\r\n",caps.AlphaCmpCaps);

//	DWORD				ShadeCaps;
	fprintf(f,"ShadeCaps\r\n");
	if(caps.ShadeCaps&D3DPSHADECAPS_ALPHAGOURAUDBLEND )	fprintf(f,"D3DPSHADECAPS_ALPHAGOURAUDBLEND : \tYES\r\n");
	else										fprintf(f,"D3DPSHADECAPS_ALPHAGOURAUDBLEND : \tNO\r\n");
	if(caps.ShadeCaps&D3DPSHADECAPS_COLORGOURAUDRGB )	fprintf(f,"D3DPSHADECAPS_COLORGOURAUDRGB : \tYES\r\n");
	else										fprintf(f,"D3DPSHADECAPS_COLORGOURAUDRGB : \tNO\r\n"); 
	if(caps.ShadeCaps&D3DPSHADECAPS_FOGGOURAUD )	fprintf(f,"D3DPSHADECAPS_FOGGOURAUD : \t\tYES\r\n");
	else										fprintf(f,"D3DPSHADECAPS_FOGGOURAUD : \t\tNO\r\n");
	if(caps.ShadeCaps&D3DPSHADECAPS_SPECULARGOURAUDRGB )	fprintf(f,"D3DPSHADECAPS_SPECULARGOURAUDRGB : \tYES\r\n");
	else										fprintf(f,"D3DPSHADECAPS_SPECULARGOURAUDRGB : \tNO\r\n");
//	DWORD				TextureCaps;
	fprintf(f,"TextureCaps\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_ALPHA )	fprintf(f,"D3DPTEXTURECAPS_ALPHA : \t\tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_ALPHA : \t\tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_ALPHAPALETTE )	fprintf(f,"D3DPTEXTURECAPS_ALPHAPALETTE : \t\tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_ALPHAPALETTE : \t\tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_CUBEMAP )	fprintf(f,"D3DPTEXTURECAPS_CUBEMAP : \t\tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_CUBEMAP : \t\tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_CUBEMAP_POW2 )	fprintf(f,"D3DPTEXTURECAPS_CUBEMAP_POW2 : \t\tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_CUBEMAP_POW2 : \t\tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_MIPCUBEMAP )	fprintf(f,"D3DPTEXTURECAPS_MIPCUBEMAP : \t\tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_MIPCUBEMAP : \t\tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_MIPMAP )	fprintf(f,"D3DPTEXTURECAPS_MIPMAP : \t\tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_MIPMAP : \t\tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_MIPVOLUMEMAP )	fprintf(f,"D3DPTEXTURECAPS_MIPVOLUMEMAP : \t\tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_MIPVOLUMEMAP : \t\tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_NONPOW2CONDITIONAL )	fprintf(f,"D3DPTEXTURECAPS_NONPOW2CONDITIONAL : \tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_NONPOW2CONDITIONAL : \tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_PERSPECTIVE )	fprintf(f,"D3DPTEXTURECAPS_PERSPECTIVE : \t\tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_PERSPECTIVE : \t\tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_POW2 )	fprintf(f,"D3DPTEXTURECAPS_POW2 : \t\t\tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_POW2 : \t\t\tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_PROJECTED )	fprintf(f,"D3DPTEXTURECAPS_PROJECTED : \t\tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_PROJECTED : \t\tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_SQUAREONLY )	fprintf(f,"D3DPTEXTURECAPS_SQUAREONLY : \t\tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_SQUAREONLY : \t\tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_TEXREPEATNOTSCALEDBYSIZE )	fprintf(f,"D3DPTEXTURECAPS_TEXREPEATNOTSCALEDBYSIZE : YES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_TEXREPEATNOTSCALEDBYSIZE : NO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_VOLUMEMAP )	fprintf(f,"D3DPTEXTURECAPS_VOLUMEMAP : \t\tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_VOLUMEMAP : \t\tNO\r\n");
	if(caps.TextureCaps&D3DPTEXTURECAPS_VOLUMEMAP_POW2 )	fprintf(f,"D3DPTEXTURECAPS_VOLUMEMAP_POW2 : \tYES\r\n");
	else										fprintf(f,"D3DPTEXTURECAPS_VOLUMEMAP_POW2 : \tNO\r\n");

//	DWORD				TextureFilterCaps;
//	DWORD				CubeTextureFilterCaps;
//	DWORD				VolumeTextureFilterCaps;

	fprintf(f,"FilterCaps \t\t\t\tTexture\tCube\tVolume\r\n");
		fprintf(f,"D3DPTFILTERCAPS_MAGFAFLATCUBIC :\t");
	if(caps.TextureFilterCaps&		D3DPTFILTERCAPS_MAGFAFLATCUBIC )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.CubeTextureFilterCaps&	D3DPTFILTERCAPS_MAGFAFLATCUBIC )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureFilterCaps&D3DPTFILTERCAPS_MAGFAFLATCUBIC )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
		fprintf(f,"D3DPTFILTERCAPS_MAGFANISOTROPIC :\t");
	if(caps.TextureFilterCaps&		D3DPTFILTERCAPS_MAGFANISOTROPIC )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.CubeTextureFilterCaps&	D3DPTFILTERCAPS_MAGFANISOTROPIC )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureFilterCaps&D3DPTFILTERCAPS_MAGFANISOTROPIC )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
		fprintf(f,"D3DPTFILTERCAPS_MAGFLINEAR :\t\t");
	if(caps.TextureFilterCaps&		D3DPTFILTERCAPS_MAGFLINEAR )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.CubeTextureFilterCaps&	D3DPTFILTERCAPS_MAGFLINEAR )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureFilterCaps&D3DPTFILTERCAPS_MAGFLINEAR )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
		fprintf(f,"D3DPTFILTERCAPS_MAGFPOINT :\t\t");
	if(caps.TextureFilterCaps&		D3DPTFILTERCAPS_MAGFPOINT )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.CubeTextureFilterCaps&	D3DPTFILTERCAPS_MAGFPOINT )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureFilterCaps&D3DPTFILTERCAPS_MAGFPOINT )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
		fprintf(f,"D3DPTFILTERCAPS_MINFANISOTROPIC :\t");
	if(caps.TextureFilterCaps&		D3DPTFILTERCAPS_MINFANISOTROPIC )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.CubeTextureFilterCaps&	D3DPTFILTERCAPS_MINFANISOTROPIC )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureFilterCaps&D3DPTFILTERCAPS_MINFANISOTROPIC )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
		fprintf(f,"D3DPTFILTERCAPS_MINFLINEAR :\t\t");
	if(caps.TextureFilterCaps&		D3DPTFILTERCAPS_MINFLINEAR )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.CubeTextureFilterCaps&	D3DPTFILTERCAPS_MINFLINEAR )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureFilterCaps&D3DPTFILTERCAPS_MINFLINEAR )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
		fprintf(f,"D3DPTFILTERCAPS_MINFPOINT :\t\t");
	if(caps.TextureFilterCaps&		D3DPTFILTERCAPS_MINFPOINT )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.CubeTextureFilterCaps&	D3DPTFILTERCAPS_MINFPOINT )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureFilterCaps&D3DPTFILTERCAPS_MINFPOINT )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
		fprintf(f,"D3DPTFILTERCAPS_MIPFLINEAR :\t\t");
	if(caps.TextureFilterCaps&		D3DPTFILTERCAPS_MIPFLINEAR )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.CubeTextureFilterCaps&	D3DPTFILTERCAPS_MIPFLINEAR )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureFilterCaps&D3DPTFILTERCAPS_MIPFLINEAR )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
		fprintf(f,"D3DPTFILTERCAPS_MIPFPOINT :\t\t");
	if(caps.TextureFilterCaps&		D3DPTFILTERCAPS_MIPFPOINT )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.CubeTextureFilterCaps&	D3DPTFILTERCAPS_MIPFPOINT )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureFilterCaps&D3DPTFILTERCAPS_MIPFPOINT )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");

//	DWORD				TextureAddressCaps;
//	DWORD				VolumeTextureAddressCaps;

	fprintf(f,"AddressCaps \t\t\t\tTexture\tVolume\r\n");
		fprintf(f,"D3DPTADDRESSCAPS_BORDER :\t\t");
	if(caps.TextureAddressCaps&		 D3DPTADDRESSCAPS_BORDER )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureAddressCaps&D3DPTADDRESSCAPS_BORDER )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
	fprintf(f,"D3DPTADDRESSCAPS_CLAMP :\t\t");
	if(caps.TextureAddressCaps&		 D3DPTADDRESSCAPS_CLAMP )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureAddressCaps&D3DPTADDRESSCAPS_CLAMP )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
	fprintf(f,"D3DPTADDRESSCAPS_INDEPENDENTUV :\t");
	if(caps.TextureAddressCaps&		 D3DPTADDRESSCAPS_INDEPENDENTUV )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureAddressCaps&D3DPTADDRESSCAPS_INDEPENDENTUV )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
	fprintf(f,"D3DPTADDRESSCAPS_MIRROR :\t\t");
	if(caps.TextureAddressCaps&		 D3DPTADDRESSCAPS_MIRROR )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureAddressCaps&D3DPTADDRESSCAPS_MIRROR )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
	fprintf(f,"D3DPTADDRESSCAPS_MIRRORONCE :\t\t");
	if(caps.TextureAddressCaps&		 D3DPTADDRESSCAPS_MIRRORONCE )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureAddressCaps&D3DPTADDRESSCAPS_MIRRORONCE )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
	fprintf(f,"D3DPTADDRESSCAPS_WRAP :\t\t\t");
	if(caps.TextureAddressCaps&		 D3DPTADDRESSCAPS_WRAP )fprintf(f,"YES\t");	else fprintf(f,"NO\t");
	if(caps.VolumeTextureAddressCaps&D3DPTADDRESSCAPS_WRAP )fprintf(f,"YES\r\n");	else fprintf(f,"NO\r\n");
 
	//	DWORD				LineCaps;
	fprintf(f,"LineCaps 5: \t\t\t\t%x\r\n",caps.LineCaps);

//	float				ExtentsAdjust;
	fprintf(f,"ExtentsAdjust : \t\t\t%f\r\n",caps.ExtentsAdjust);
//	DWORD				StencilCaps;
	fprintf(f,"StencilCaps : \t\t\t\t%x\r\n",caps.StencilCaps);

//	DWORD				FVFCaps;
	if(caps.FVFCaps&D3DFVFCAPS_DONOTSTRIPELEMENTS )	fprintf(f,"D3DFVFCAPS_DONOTSTRIPELEMENTS : \tYES\r\n");
	else										fprintf(f,"D3DFVFCAPS_DONOTSTRIPELEMENTS : \tNO\r\n");
	if(caps.FVFCaps&D3DFVFCAPS_PSIZE )	fprintf(f,"D3DFVFCAPS_PSIZE : \t\t\tYES\r\n");
	else										fprintf(f,"D3DFVFCAPS_PSIZE : \t\t\tNO\r\n");
	if(caps.FVFCaps&D3DFVFCAPS_TEXCOORDCOUNTMASK )	fprintf(f,"D3DFVFCAPS_TEXCOORDCOUNTMASK : \t\tYES\r\n");
	else										fprintf(f,"D3DFVFCAPS_TEXCOORDCOUNTMASK : \t\tNO\r\n");

//	DWORD				VertexProcessingCaps;	
	fprintf(f,"VertexProcessingCaps\r\n");
	if(caps.VertexProcessingCaps&D3DVTXPCAPS_DIRECTIONALLIGHTS )	fprintf(f,"D3DVTXPCAPS_DIRECTIONALLIGHTS : \tYES\r\n");
	else															fprintf(f,"D3DVTXPCAPS_DIRECTIONALLIGHTS : \tNO\r\n");
	if(caps.VertexProcessingCaps&D3DVTXPCAPS_LOCALVIEWER )	fprintf(f,"D3DVTXPCAPS_LOCALVIEWER : \t\tYES\r\n");
	else													fprintf(f,"D3DVTXPCAPS_LOCALVIEWER : \t\tNO\r\n");
	if(caps.VertexProcessingCaps&D3DVTXPCAPS_MATERIALSOURCE7 )	fprintf(f,"D3DVTXPCAPS_MATERIALSOURCE7 : \t\tYES\r\n");
	else														fprintf(f,"D3DVTXPCAPS_MATERIALSOURCE7 : \t\tNO\r\n");
	if(caps.VertexProcessingCaps&D3DVTXPCAPS_POSITIONALLIGHTS )	fprintf(f,"D3DVTXPCAPS_POSITIONALLIGHTS : \t\tYES\r\n");
	else														fprintf(f,"D3DVTXPCAPS_POSITIONALLIGHTS : \t\tNO\r\n");
	if(caps.VertexProcessingCaps&D3DVTXPCAPS_TEXGEN )	fprintf(f,"D3DVTXPCAPS_TEXGEN : \t\t\tYES\r\n");
	else												fprintf(f,"D3DVTXPCAPS_TEXGEN : \t\t\tNO\r\n");
	if(caps.VertexProcessingCaps&D3DVTXPCAPS_TWEENING )	fprintf(f,"D3DVTXPCAPS_TWEENING : \t\t\tYES\r\n");
	else												fprintf(f,"D3DVTXPCAPS_TWEENING : \t\t\tNO\r\n");
	if(caps.VertexProcessingCaps&D3DVTXPCAPS_NO_VSDT_UBYTE4 )	fprintf(f,"D3DVTXPCAPS_NO_VSDT_UBYTE4 : \t\tYES\r\n");
	else														fprintf(f,"D3DVTXPCAPS_NO_VSDT_UBYTE4 : \t\tNO\r\n");

//	float				GuardBandLeft;
	fprintf(f,"GuardBandLeft : \t%f\r\n",caps.GuardBandLeft);
//	float				GuardBandTop;
	fprintf(f,"GuardBandTop : \t\t%f\r\n",caps.GuardBandTop);
//	float				GuardBandRight;
	fprintf(f,"GuardBandRight : \t%f\r\n",caps.GuardBandRight);
//	float				GuardBandBottom;
	fprintf(f,"GuardBandBottom : \t%f\r\n",caps.GuardBandBottom);

	fclose(f);
}

void Scene::SaveScreen_BMP()
{
	int scr_size_x; int scr_size_y;
	IDirect3DSurface8 *sur;
	D3DDISPLAYMODE Mode;
	
	pd->GetDisplayMode( &Mode);
	Mode.Format = D3DFMT_A8R8G8B8;
	if(pd->CreateImageSurface(Mode.Width, Mode.Height, Mode.Format, &sur)!=D3D_OK )return;
	
	D3DSURFACE_DESC surd;
	if(pd->GetFrontBuffer( sur)!=D3D_OK){sur->Release(); return;}
	sur->GetDesc( &surd);

	D3DLOCKED_RECT LockedRect;
	if(sur->LockRect( &LockedRect, NULL, D3DLOCK_READONLY)!=D3D_OK)
	{
		sur->UnlockRect();
		sur->Release();
		return;
	}

	scr_size_x = surd.Width;
	scr_size_y = surd.Height;

	BITMAPFILEHEADER fileheader;
	BITMAPINFOHEADER infoheader;

	FILE *sub;
	int line_size = 3*scr_size_x;
	if(line_size%4)line_size += 4 - line_size%4;

	unsigned char* buf = new unsigned char[3*line_size*scr_size_y];
	char file[30];

	if(buf==NULL )
	{
		sur->UnlockRect();
		sur->Release();
		return;
	}

	int Pitch = LockedRect.Pitch;
	unsigned char* p = (unsigned char*) LockedRect.pBits;
	for(unsigned int y=0; y<surd.Height; y++)
	{
		for(unsigned int x=0; x<surd.Width; x++)
		{
			buf[line_size*y+3*x+0] = p[Pitch*(surd.Height-y-1)+4*x+0];
			buf[line_size*y+3*x+1] = p[Pitch*(surd.Height-y-1)+4*x+1];
			buf[line_size*y+3*x+2] = p[Pitch*(surd.Height-y-1)+4*x+2];
		}
	}
	sur->UnlockRect();
	sur->Release();

	for(int c=0;c<999;c++)
	{
		char pom[20];
		strcpy(file,"screen");
		if(c<100)strcat(file,"0");
		if(c<10)strcat(file,"0");
		strcat(file,itoa(c,pom,10));
		strcat(file,".bmp");
		sub = fopen(file,"r");
		if(sub==NULL)break;
		if(sub!=NULL)fclose(sub);
	}
	
	fileheader.bfType = 0x4D42; // Magic identifier   - "BM"	| identifikacia BMP suboru musi byt "BM"
	fileheader.bfSize = 3*scr_size_x*scr_size_y+sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER);	// File size in bytes			| velkos suboru v byte
	fileheader.bfReserved1 = 0;
	fileheader.bfReserved2 = 0;
	fileheader.bfOffBits = sizeof(BITMAPFILEHEADER)+sizeof(BITMAPINFOHEADER);	// Offset to image data, bytes	| posun na zaciatok dat
	
	infoheader.biSize = sizeof(BITMAPINFOHEADER);	// Header size in bytes			| velkost hlavicky BITMAPINFOHEADER
	infoheader.biWidth = scr_size_x;	// Width of image			| sirka obrazka - sizeX
	infoheader.biHeight = scr_size_y;	// Height of image			| vyska obrazka - sizeY
	infoheader.biPlanes = 1;		// Number of colour planes	| pocet farebnych rovin musi bit 1
	infoheader.biBitCount = 24;		// Bits per pixel			| bitov na pixel moze bit 1,4,8,24
	infoheader.biCompression = 0;	// Compression type			| typ compresie , 0 - bez kompresie
	infoheader.biSizeImage = line_size*infoheader.biHeight ;	// Image size in bytes		| velkost obrazka v byte
	infoheader.biXPelsPerMeter = 0;	// Pixels per meter X		| pixelov na meter v smere x
	infoheader.biYPelsPerMeter = 0;	// Pixels per meter Y		| pixelov na meter v smere y
	infoheader.biClrUsed = 0;		// Number of colours		| pocet  farieb v palete, ak 0 vsetky su pouzivane
	infoheader.biClrImportant = 0;	// Important colours		| dolezite farby v palete, ak 0 vsetky su dolezite
	
	sub = fopen(file,"wb");
	fwrite( &fileheader, sizeof(BITMAPFILEHEADER), 1, sub);
	fwrite( &infoheader, sizeof(BITMAPINFOHEADER), 1, sub);
	fwrite( buf, line_size*scr_size_y, 1, sub);

	delete[] buf;
	fclose(sub);
}
